#include <SoftwareSerial.h>

/*
 * 
 *  Arduino example for the Super8 Relay Module
 *  Modified 11-June-2017
 *  
 *  (c)www.tctec.net
 *  
 *  Code provided 'as is' no responsibility is accepted for use or misuse of this example.
 * 
 * 
 * 
 */

SoftwareSerial relayPort(10,11);  // RX, TX

// For direct control of relays with digital outputs
//
const int direct_R1 = 2;
const int direct_R2 = 3;
const int direct_R3 = 4;
const int direct_R4 = 5;
const int direct_R5 = 6;
const int direct_R6 = 7;
const int direct_R7 = 8;
const int direct_R8 = 9;


void setup() {
  Serial.begin(9600);
  while(!Serial){}

  relayPort.begin(9600);


  // setup digital outputs if using them to drive relays directly
  //
  pinMode(direct_R1, OUTPUT);
  pinMode(direct_R2, OUTPUT);
  pinMode(direct_R3, OUTPUT);
  pinMode(direct_R4, OUTPUT);
  pinMode(direct_R5, OUTPUT);
  pinMode(direct_R6, OUTPUT);
  pinMode(direct_R7, OUTPUT);
  pinMode(direct_R8, OUTPUT);

  digitalWrite(direct_R1, 0);
  digitalWrite(direct_R2, 0);
  digitalWrite(direct_R3, 0);
  digitalWrite(direct_R4, 0);
  digitalWrite(direct_R5, 0);
  digitalWrite(direct_R6, 0);
  digitalWrite(direct_R7, 0);
  digitalWrite(direct_R8, 0);
  

}




/*
//-------------------------------------------------------------------------------------------
// Function to update any number of relays on the relay bus
// Every byte that is sent to TX is recieved back on RX.
// Only the node counter byte is modified. (= the number of nodes traversed)
// Compare byte recieved with bytes sent to confirm correct reception of commands on the bus
//
// Returns:
//
// TRUE if successful
// FALSE if failed  (i.e failed error check)
//
//-------------------------------------------------------------------------------------------
*/

bool updateRelays_S8(bool states[], int numRelays)
{  

  byte bitMask = 1;
  int bitCount = 0;
  int relayByte = 0;
  byte highNibble = 0;
  byte lowNibble = 0;
  int byteCount = 0;
  byte tempByte = 0;
  int i = 0;
  int errorCount = 0;


  if ((numRelays %8 ) > 0)
  {
    // The number of relays must be a multiple of 8
    // There are 8 relays on each Super8 board
    //
    return (false);
  }
  if (numRelays <= 0)
  {
    return (false);
  }

     
  byte relayBytes[(numRelays / 8)];

  for(i = 0 ; i < numRelays; i++)
  {    
    if (states[i] == true)
    {
      relayByte |= bitMask;   // set the bit in the relayByte 
    }
    

    bitMask <<= 1;            // shift left 1 bit
    bitCount++;
    
    if (bitCount >= 8)
    {     
      // we have 8 relays states, formatted to a single byte 
      //
      relayBytes[byteCount] = relayByte;
      relayByte = 0;
      bitCount = 0;  
      bitMask = 1; 
          
      byteCount++;
    }     
  }


  // we have set up all the bytes, now send commands to the relay port
  //
  relayPort.write((byte)0xCC);     // 'Start of command' 
  getResponse();
  relayPort.write((byte)0x01);     // node counter set to 1
  getResponse();

  for(i = 0 ; i < byteCount;  i++)    // send byte for the last relay board, first.
  {
    // send the relay bytes (2 per node)
    //    
    tempByte = relayBytes[i];
    tempByte >>= 4 ;             // sending the high nibble first
    tempByte &= 0x0F;
    tempByte |= 0xA0;          // 'An' is the high nibble
    
    relayPort.write((byte)tempByte);   
    getResponse();    
    
    tempByte = relayBytes[i];    
    tempByte &= 0x0F;          // keep only the low nibble
    tempByte |= 0xB0;          // 'Bn' is the low nibble

    relayPort.write((byte)tempByte);
     getResponse();
  }

  // if we are happy that command was recieved, send command to engage new relay state
  //
  relayPort.write((byte)0xCC);     // 'Start of command' 
  getResponse();
  relayPort.write((byte)0x01);     // node counter set to 1
  getResponse();

  for(i = 0 ; i < byteCount;  i++)
  {
    relayPort.write((byte)0xDD);
    getResponse();
  }

  // check that Engage command was successfully recieved
  //

  return(true);
  
}

void getResponse()
{
  return;
  delay(1);
  while (relayPort.available())
    {
      //relayPort.read();
      Serial.print(relayPort.read(), HEX);  
      Serial.print(" ");      
    }
 
}


const int RELAY_COUNT = 8;             // boards x 8
const int BOARDS = 1;                  // how many boards in the daisy chain



// Test relay control using the relay bus (serial coms)
//
void testBus()
{

  static bool relaySettings[RELAY_COUNT];
  static int relayBoard = 0;
  static int relayToSet = 7;

  // reset relay array
  //
  for(int i = 0 ; i < RELAY_COUNT; i++)
  {
    relaySettings[i] = false;
  }
  
  relaySettings[relayToSet + (relayBoard * 8)] = true;     // turn one relay on

  updateRelays_S8(relaySettings, RELAY_COUNT);

  relayToSet--;
  if (relayToSet < 0)
  {
    relayBoard++;
    if(relayBoard > BOARDS-1)
    {
      relayBoard = 0 ;
    }
    relayToSet = 7;
  }
  
}

// Test control of relays with direct digital output
//
void testDirect(byte states)
{
  
  // Relay 1
  //
  if (states & 0b00000001)
  {
     digitalWrite(direct_R1, 1);    
  }
  else
  {
     digitalWrite(direct_R1, 0);    
  }

  // Relay 2
  //
  if (states & 0b00000010)
  {
     digitalWrite(direct_R2, 1);    
  }
  else
  {
     digitalWrite(direct_R2, 0);    
  }

  // Relay 3
  //
  if (states & 0b00000100)
  {
     digitalWrite(direct_R3, 1);    
  }
  else
  {
     digitalWrite(direct_R3, 0);    
  }

  // Relay 4
  //  
  if (states & 0b00001000)
  {
     digitalWrite(direct_R4, 1);    
  }
  else
  {
     digitalWrite(direct_R4, 0);    
  }

  // Relay 5
  //
  if (states & 0b00010000)
  {
     digitalWrite(direct_R5, 1);    
  }
  else
  {
     digitalWrite(direct_R5, 0);    
  }

  // Relay 6
  //  
  if (states & 0b00100000)
  {
     digitalWrite(direct_R6, 1);    
  }
  else
  {
     digitalWrite(direct_R6, 0);    
  }
    
  // Relay 7
  //
  if (states & 0b01000000)
  {
     digitalWrite(direct_R7, 1);    
  }
  else
  {
     digitalWrite(direct_R7, 0);    
  }

  // Relay 8
  //  
  if (states & 0b10000000)
  {
     digitalWrite(direct_R8, 1);    
  }
  else
  {
     digitalWrite(direct_R8, 0);    
  }  
  
}



void loop() {
  
  static byte states = 0;
  
  delay(100);
  //testDirect(states++);
  testBus();
  

 

  

  

}
